/*******************************************************************************

    TINY FREQUENCIMETER



    FREQUENCIMETRO EXPERIMENTAL C/ PIC12F675 E LCD (MTODO LPLEX - 3 FIOS)


    AUTOR: CLAUDIO LARIOS

    C/ XTAL DE 4 MHZ (OSCILADOR EXTERNO NO PINO 2)

    DATA:06/05/2016

    
    USO DIDTICO
    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS

*******************************************************************************/
#include <12F675.h>
#device adc=10
#use delay(clock=4000000)
#fuses NOWDT,EC, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT
#use fast_io(a)
//bytes
#byte  trisio = 0x85
#byte  gpio   = 0x05
#byte  cmcon  = 0x19
#byte  intcon = 0x0b
#byte  option_reg = 0x81
//bit
#bit t0if = 0x0b.2 //timer0 overflow flag
#bit t0ie = 0x0b.5 //on/off interrupo do tmr0
#bit t1if = 0x0c.0 //over flow tmr1
#bit t1ie = 0x8c.0 //liga interrupo do tmr1
#bit gie  = 0x0b.7 //geral int
#bit peie = 0x0b.6 //geral int perifricos

//pinos do lcd (metodo Lariosplex - 3 fios)
#bit DB76  = 0x5.0  //pino 7 db7=direto db6=resistor 10k +cap 10nF ao gnd
#bit DB54  = 0x5.1  //pino 6 db5=direto db4=resistor 10k +cap 10nF ao gnd
#bit E_RS  = 0x5.4  //pino 3 E=direto RS=resistor 10k +cap 10nF ao gnd
#bit in    = 0x5.2  //pino 5 entrada
#bit tris_in = 0x85.2 //sentido do pino 5

#define vcarga 500 // por tentativa e erro (entre 200 a 500)
#define vdescarga 500//idem
#define v_delay_clk 2  // de 2 a 50 conforme LCD

#define time_max 0XFF00  //define o tempo do time_out

//variveis globais
unsigned int1 RS;
unsigned int8 dmilh,umilh,cmil,dmil,umil,cent,dez,un,old,presc,a;
unsigned int32 res;

unsigned int16 est;

#int_global
void isr_timer0(){//interrupo do timer 0 (modo global)
   t0if=0;//apaga flag manualmente
   est++;//incrementa registrador de estouro do timer 0
}

/*******************************************************************************
                          SUBROTINAS
*******************************************************************************/


void descarregar(){ DB76=0;DB54=0;E_RS=0; delay_us(vdescarga);}//tempo de descarga
void t_carga(){delay_us(vcarga);}//tempo de carga
void clk(){ E_RS=1; delay_cycles(v_delay_clk); E_RS=0;}//gera clock par Lcd


/*******************************************************************************
                          SUBROTINAS de LCD
*******************************************************************************/


/*******************************************************************************
                         ENVIA NIBBLE
*******************************************************************************/
void env_nible(unsigned int8 nib){
    descarregar();//descarrega capacitores
    nib&=0x0f;//limita
    if(nib&0x01)DB54=1;//testa bit 0
    if(nib&0x04)DB76=1;//testa bit 2
    if(RS)E_RS=1;else E_RS=0;//comando ou dado
    t_carga();
    if(nib&0x02)DB54=1; else DB54=0;//testa bit 1
    if(nib&0x08)DB76=1; else DB76=0;//testa bit 3
    clk();//clock

}

/*******************************************************************************
                          ENVIA DADOS PARA O LCD
*******************************************************************************/

void env_lcd_d(unsigned int8 dado){

 RS=1;//indica que enviar 'dados'
 env_nible(dado>>4);//high nibble primeiro
 env_nible(dado);//low nibble
 delay_ms(4);//delay
}

/*******************************************************************************
                         ENVIA COMANDOS PARA LCD
*******************************************************************************/
void env_lcd_c(unsigned int8 comando){

 RS=0;//indica que enviar 'comandos'
 env_nible(comando>>4);//high nibble primeiro
 env_nible(comando);//low nibble
 delay_ms(4);//delay
}

/*******************************************************************************
                         INICIALIZAO DO LCD
*******************************************************************************/

void Lcd_Init(){
  unsigned int8 a;

   descarregar();//descarrega capacitores
   RS=0; //enviar comando
   delay_ms(40);//delay inicial
   for(a=0;a<3;a++) { env_nible(3);delay_ms(5);}//envia 3x 0x03
   env_nible(2);//envia 0x02
   delay_ms(5);//delay 5 ms
   env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
   env_lcd_c(0x06);// no shift
   env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
   env_lcd_c(0x01);//limpa tela
}



/*******************************************************************************
               MOSTRA NO LCD A LEITURA PROCESSADA
*******************************************************************************/
void mostra_res(){
  unsigned int1 zesq=0;


     if(!dmilh){dmilh=240;}else{ zesq=1;}//apaga zeros a esquerda
     env_lcd_d(dmilh+'0');//envia dezena de milhar
     if(!umilh&&!zesq){umilh=240;}else{ zesq=1;}//apaga zeros a esquerda
     env_lcd_d(umilh+'0');//envia dezena de milhar
     if(zesq)env_lcd_d('.');else env_lcd_d(' ');
     if(!cmil&&!zesq){cmil=240;}else{ zesq=1;}//apaga zeros a esquerda
     env_lcd_d(cmil+'0');//envia dezena de milhar
     if(!dmil&&!zesq){dmil=240;}else{ zesq=1;}//apaga zeros a esquerda
     env_lcd_d(dmil+'0');//envia dezena de milhar
     if(!umil&&!zesq){umil=240;}else{ zesq=1;}
     env_lcd_d(umil+'0');//envia milhar
     if(zesq)env_lcd_d('.');else env_lcd_d(' ');
     if(!cent&&!zesq){cent=240;}else{ zesq=1;}
     env_lcd_d(cent+'0');//envia centena
     if(!dez&&!zesq){dez=240;}else{zesq=1;}
     env_lcd_d(dez+'0');//envia dezena
     env_lcd_d(un+'0');//envia unidade
     env_lcd_d(' ');//espao
}

/*******************************************************************************
                 CONVERTE DE HEXA PARA DECIMAL
*******************************************************************************/
void converte_res(){

     dmilh=res/10000000;//obtm dezena de milhar
     res=res%10000000;//coloca o resto da conta acima
     umilh=res/1000000;//obtm dezena de milhar
     res=res%1000000;//coloca o resto da conta acima
     cmil=res/100000;//obtm dezena de milhar
     res=res%100000;//coloca o resto da conta acima
     dmil=res/10000;//obtm dezena de milhar
     res=res%10000;//coloca o resto da conta acima
     umil=res/1000;//obtem a unidade de milhar
     res=res%1000;//coloca o resto da conta acima
     cent= res/100;//obtem a centena
     res=res%100;//coloca o resto da conta acima
     dez=res/10;//obtem a dezena
     un=res%10;//coloca o resto da conta acima (obtem a unidade)

}


/*******************************************************************************
                        ROTINA MAIN
*******************************************************************************/

void main() {


   setup_adc_ports(NO_ANALOGS|VSS_VDD);//libera portas
   setup_adc(ADC_OFF);//desliga conversor analgico/digital
   setup_timer_0 (RTCC_DIV_256 |RTCC_EXT_L_TO_H);
   setup_vref(FALSE);//desliga vref
   cmcon=7;//desliga comparador interno
   trisio=0b101100;//ajusta entradas e sadas
   gpio=0;//zera tudo
   descarregar();//descarga dos capacitores do Lariosplex
   Lcd_Init();//inicializa Lcd
   printf(env_lcd_d("Tiny Freq_meter"));//mostra o nome
   env_lcd_c(0xc0);//linha 2
   printf(env_lcd_d("Larios.Tecnol.Ws"));// o blog
   delay_ms(2000);//tempo de apresentao do nome no LCD
   env_lcd_c(1);//apaga tela
   env_lcd_c(0x80);//linha 1
   printf(env_lcd_d("Tiny Freq_meter"));//mostra o nome
   env_lcd_c(0xc9); // 6 posico da linha 2
   printf(env_lcd_d(" 0 Hz"));//zeragem inicial
   gie=1;//liga interrupes

/*******************************************************************************
           LOOP PRINCIPAL DE REPETIO
*******************************************************************************/


   for(;;){


/*******************************************************************************
        ROTINA DE OPERAO DO FREQUENCMETRO
*******************************************************************************/
     est=0;//reseta contador de overflow do timer 0
     set_timer0(0); //zera timer e prescaller
     t0if=0;//apaga flag do timer 0
     enable_interrupts(INT_TIMER0);
     set_timer0(0); //zera timer e prescaller
     tris_in=1;//entrada 
     delay_ms(1000);//le a frequencia por 1 segundo
     tris_in=0;//sada (para impedir a contagem do timer 0)
     disable_interrupts(INT_TIMER0);//desliga interrupes
     old=get_timer0();//le o valor do timer 0
     res=256* old;//salva na resultado  
     res+=est*65536;//soma nmero de overflows do timer 0

     presc=255;//carga inicial
     while(old==get_timer0()){//obtm o valor do prescaller
       presc--;//decrementa
       setup_timer_0 (RTCC_DIV_256 |RTCC_EXT_H_TO_L);//pulsa prescaller
       setup_timer_0 (RTCC_DIV_256 |RTCC_EXT_L_TO_H);
     }
     res+=presc;// o valor obtido do prescaller

     converte_res();// ajusta para decimal
     //envia resultado para lcd
     env_lcd_c(0xc1); // 2 posico da linha 2
     mostra_res(); //mostra resultado no LCD
     printf(env_lcd_d("Hz "));//envia 'rpm'


   }//for(;;)

}//void main()
